from django.db import models


class SingletonCrfModelMixin(models.Model):
    """Enforces one record per subject.

    Requires CrfModelMixn.
    """

    singleton_field = models.CharField(
        verbose_name="subject identifier",
        max_length=50,
        unique=True,
        help_text="auto updated for unique constraint",
        null=True,
        editable=False,
    )

    def save(self, *args, **kwargs):
        # enforce singleton constraint on instance, 1 per subject
        self.singleton_field = self.related_visit.subject_identifier
        super().save(*args, **kwargs)

    class Meta:
        abstract = True



from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_model import model_exists_or_raise


def raise_if_crf_does_not_exist(subject_visit, model: str) -> None:
    model_cls = django_apps.get_model(model)
    try:
        model_exists_or_raise(subject_visit=subject_visit, model_cls=model_cls)
    except ObjectDoesNotExist:
        raise forms.ValidationError(f"Complete {model_cls._meta.verbose_name} CRF first.")
